// decoder.h
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module contains interface of class Decoder using for decoding a section
//

#ifndef decoderH
#define decoderH

#include "udaclasses.h"
#include "shared.h"
#include "instrset.h"
#include "sections.h"

//! Disassemble instructions
#define DA_INSTRUCTIONS 0x01
//! When new label in disassembling, create it
#define DA_NEW_LABELS 0x02
//! Disassemble with labels
#define DA_WITH_LABELS 0x04

//! DecoderException - exception thrown while decode
class DecoderException;

//! Decoder - no comment
class DecoderException : public ::Exception {
public:
    DecoderException(string s) : Exception(s) {}
    virtual ~DecoderException() {}
    virtual void Print() { cerr<<"Decoder error: "<<value<<endl; }
};

//! Parameters for decoder
struct DecoderInfo {
    //! decoded section
    Section *section;
    //! binary file with all sections
    BinaryFile *binfile;
    //! starting address (default=0)
    Address start;
    //! stopping address (it is not decoded) (default=0)
    Address stop;
    //! size in bytes to decode (default=0)
    unsigned size;
    //! print to standart output (default=true)
    bool print;
    //! print address (default=true)
    bool address;
    //! print hex dump (default=true)
    bool hexdump;
    //! print symbolic instruction (default=true)
    bool symbolic;
    //! in symbolic address print only symbolic name (default=false)
    bool onlyname; 
    //! scan immediate data for addresses (default=true)
    bool scanaddress;
    //! Creates default parameters
    DecoderInfo();
    DecoderInfo(const DecoderInfo &decinfo) { assign(decinfo); }
    DecoderInfo &operator =(const DecoderInfo &decinfo) { return assign(decinfo); }
    DecoderInfo &assign(const DecoderInfo &decinfo);
};

class Decoder {
    //prints line
    void print_line(Address addr, unsigned size, string &symbolic_instruction);
    //hexadecimal output of section
    void HexOut();
public:
    //! Decode the section, if print is false nothing will be on output
    void Decode();
};


extern const char *StartingTable;
//! Decoder - singleton
extern Decoder decoder;
extern DecoderInfo decinfo;

#endif
