// plugins.h
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module contains interface of plugins (loading, syncing).
//

#ifndef pluginsH
#define pluginsH

#include <string>
#include <vector>
using namespace std;

#include "udaclasses.h"
#include "shared.h"

extern const char *pluginsdir; // dir with plugins
extern const char *configfile; // file with list of plugins

////////////////////////////////////////////////////////////
//Events for plugin

//! return non zero on success
typedef int (*Ev_IsFileDefined)(const char*);
//! return number of loaded sections
typedef int (*Ev_LoadSections)(BinFileData &);
//! return number of loaded symbols
typedef int (*Ev_LoadSymbols)(AbstractSymbols &);

int EvEmpty_data(void*);
int EvEmpty_void();

////////////////////////////////////////////////////////////
//! Plugin information
class Plugin {
public:
    string filename; //base filename
    void *dynfile; //descriptor to opened plugin
    int type; //type of plugin PLUGIN_SECTIONS or PLUGIN_SYMTABLE
    const char *ident; //text identification of plugin
    const char *syntax; //plugins parameters
    Ev_IsFileDefined IsFileDefined;
    Ev_LoadSections LoadSections;
    Ev_LoadSymbols LoadSymbols;
    Plugin();
    ~Plugin();
    //! Load plugin information. Returns 0 on success.
    int Open(const char *filename);
    //! Close plugin
    void Close();
    //! Print plugin information
    void Info();
};

typedef vector<Plugin*> VPlugins;
extern VPlugins plugins;

//! Load all plugins from plugins/ directory. Returns number of loaded plugins
int OpenPlugins();
//! Close all plugins
void ClosePlugins();
//! Print plugins informations
void PluginsInfo();

#endif
