// binonly.cc
//
// plugin for uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This library is licensed under GNU Library GPL. See the file COPYING.
//
// This library makes possible to read binary only format.
//

#include <iostream>
#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "shared.h"
using namespace std;

#define pluginname "binonly.so"

Parameters *Params;

extern const char *_syntax;

extern "C" {

void plugin_init(int &type, const char *&ident, const char *&syntax, Parameters *p)
{
    Params=p;
    type=PLUGIN_SECTIONS;
    ident="Universal disassembler plugin for binary only format";
    syntax=_syntax;
}

int is_file_defined(const char *file)
{
    debug(pluginname<<": is_file_defined...\n");
    int result=pdefined("-bof");
    if (result)
        debug(pluginname<<": is_file_defined...yes\n");
    else
        debug(pluginname<<": is_file_defined...no\n");
    return result;
}

unsigned hex2uint(const string hex)
{
    const char *c=hex.c_str();
    if (hex.substr(0, 2)=="0x")
        c+=2;
    unsigned result=0;
    while (*c)
    {
        if (isdigit(*c))
            result=result<<4 | (*c-'0');
        else
        if (isxdigit(*c))
            result=result<<4 | (lower(*c)-'a'+10);
        else
            return 0;
        c++;
    }
    return result;
}

unsigned number2uint(const string num)
{
    const char *c=num.c_str();
    if (num.substr(0, 2)=="0x")
        return hex2uint(num);
    unsigned result=0;
    while (*c)
    {
        if (isdigit(*c))
            result=result*10 + (*c-'0');
        else
            return 0;
        c++;
    }
    return result;
}

int load_sections(BinFileData &bfdata)
{
    debug(pluginname<<": load_sections...\n");
    bfdata.entry=number2uint((*Params)["-bofentry"]);
    int in=open(bfdata.name.c_str(), O_RDONLY);
    if (in!=-1)
    {
        struct stat buf;
        if (fstat(in, &buf))
        {
            close(in);
            return 0;
        }
        SectionData *sd = new SectionData;
        sd->addr=bfdata.entry;
        sd->size=buf.st_size;
        sd->executable=!pdefined("-bofnoexec");
        sd->name=".bof";
        sd->content = new byte[sd->size];
        read(in, sd->content, sd->size);
        close(in);
        bfdata.Sections.push_back(sd);
        debug(pluginname<<": load_sections "<<1<<"...ok\n");
        return 1;
    }
    return 0;
}

} // extern "C"

const char *_syntax = "\n\
    -bof            Activate this plugin for any binary file. Load all binary\n\
                    into one section .bof\n\
    -bofentry address\n\
                    set entry of section .bof\n\
    -bofnoexec      unset flag executable at section .bof";
