// sections.cc
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module implements BinaryFile and Section class. It is used as
// gateway to binary file.
//

#include <iostream> //debug()
#include "sections.h"
#include "udaclasses.h"
#include "plugins.h"
using namespace std;

///////////////////////////////////////////////////////////////////////////////////////////
// BinaryFile

BinaryFile::BinaryFile(string fname)
{
    bfdata.name=fname;
}

BinaryFile::~BinaryFile()
{
    Free();
}

// return value: number of loaded sections
int BinaryFile::Init()
{
    int numsec=0;
    VPlugins::iterator i;
    for (i=plugins.begin();i!=plugins.end();i++)
        if ((*i)->IsFileDefined(bfdata.name.c_str()))
        {
            numsec=(*i)->LoadSections(bfdata);
            break;
        }
    typedef MSections::value_type vt;
    for (int i=0;i<numsec;i++)
        sections.insert(vt(bfdata.Sections[i]->name, Section(bfdata.Sections[i])));
    return numsec;
}

// true if address is valid in any scanned section
bool BinaryFile::ScanAddress(Address addr)
{
    MSections::iterator i;
    for (i=sections.begin();i!=sections.end();i++)
        if (i->second.data->scanaddress && i->second.ValidAddress(addr))
            return true;
    return false;
}

// Free all sections
void BinaryFile::Free()
{
    debug("BinaryFile:freeing all sections\n");
    while (bfdata.Sections.size())
    {
        delete bfdata.Sections[0];
        bfdata.Sections.erase(bfdata.Sections.begin());
    }
}

// indexing section with error detection
Section &BinaryFile::operator [](const string index)
{
    MSections::iterator i=sections.find(index);
    if (i!=sections.end())
        return i->second;
    else
        throw Exception(string("section ")+index+" does not exists");
}

///////////////////////////////////////////////////////////////////////////////////////////
// Section

Section::Section(SectionData *sec)
{
    data=sec;
}

bool Section::ValidAddress(Address a)
{
    return (data!=NULL)&&
           (data->content!=NULL)&&
           (a >= data->addr)&&
           (a < data->addr+data->size);
}

byte &Section::operator [](Address a)
{
    if (ValidAddress(a))
        if (data!=NULL)
            return data->content[a-data->addr];
        else
            throw Exception(string("bad address ")+uint2hex(a)+" in section "+data->name);
    else
        throw Exception(string("bad address ")+uint2hex(a)+" in section "+data->name);
}
