// sections.h
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module is an interface for BinaryFile and Section class. It is used as
// gateway to binary file.
//

#ifndef sectionsH
#define sectionsH

#include <map>
#include <string>
#include "udaclasses.h"
using namespace std;

#include "plugins.h"
#include "shared.h"

class BinaryFile;
class Section;

typedef map<string, Section> MSections;

//! Binary File - base of a source file
class BinaryFile {
    BinaryFile();
public:
    BinFileData bfdata;
    MSections sections;

    BinaryFile(string Name);
    ~BinaryFile();
    /*! Looks for associated plugin and load all sections.
        Returns number of loaded sections.*/
    int Init();
    //! true if address is valid in any scanned section (section wich has enabled scanaddress)
    bool ScanAddress(Address addr);
    //! Free all sections
    void Free();
    Section &operator [](const string index);
};

//! Class used for disassembling. (Adapter for Decoder)
class Section {
public:
    SectionData *data;
    Section() { data=NULL; }
    Section(SectionData *sec);

    //! true if this section is defined
    bool defined() { return data!=NULL; }
    //! true if address is valid
    bool ValidAddress(Address addr);
    //! Addressing operator
    byte &operator [](Address addr);
    SectionData *operator ->() { return data; }
    operator SectionData&() { return *data; }
};

#endif
