// shared.h
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This header file is shared for program uda and their plugins.
//

#ifndef sharedH
#define sharedH

#include <map>
#include <vector>
#include <string>
#include <iostream>
#include <iomanip>
#include <libgen.h>
using namespace std;

#define PLUGIN_SECTIONS 0x01
#define PLUGIN_SYMTABLE 0x02

////////////////////////////////////////////////////////////
//base types

//! Address is a type for addressing
typedef unsigned int Address;
//! Byte is a type of a single byte
typedef unsigned char byte;

////////////////////////////////////////////////////////////
//input/output files

/*struct Files {
    char *outputfile;
    char *mapfile;
    char *binaryfile;
};*/

typedef map<string, string> Parameters;

//! Parameters map
extern Parameters *Params;

//! Returns true if parameter p is defined
#define pdefined(p) (Params->find(p)!=Params->end())

//! Print verbose information (-v parameter)
#define verbose(t) if(pdefined("-v") || pdefined("--debug")) cerr<<dec<<setfill(' ')<<t;else

//! Print debug information (--debug parameter)
#define debug(t) if(pdefined("--debug")) cerr<<dec<<setfill(' ')<<t;else

#define lower(c) (c>='A'&&c<='Z'?c+'a'-'A':c)

typedef void (*Splugin_init)(int &type, const char *&ident, const char *&syntax, Parameters *p);

////////////////////////////////////////////////////////////
//Sections

//! A section object including only data
struct SectionData {
    Address addr; //address of start
    unsigned size; //section size
    bool executable; //true if executable
    byte *content; //data
    string name; //section name
    bool scanaddress; //true=scan immediate data for addresses at this section
    SectionData() { addr=size=0; content=NULL; executable=scanaddress=false; }
};

//! Binary File Data. Used by Ev_GetAllSections.
struct BinFileData {
    //! file name
    string name;
    //! entry point
    Address entry;
    //! array of all sections in this file
    vector<SectionData*> Sections;
};


////////////////////////////////////////////////////////////
//SymbolTable

//! A component of AbstractSymbols. Map of all symbols.
typedef map<Address, string> MSymbols;

//! Proxy. Used for share a symbol table. Plugin creates this Symbols uses it.
class AbstractSymbols {
public:
    //! Symbols map. Address gets string.
    MSymbols symbols;
    BinFileData &bfdata;
    AbstractSymbols(BinFileData &bfd) : bfdata(bfd) {}
    virtual ~AbstractSymbols(){}
    //! Loads a symbol table. Virtual because of plugin don't implement it.
//    virtual void Load(const char *file) {}
};

#endif
