// symtable.cc
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module contains implementation of class Symbols - administration of symbols
//

#include "symtable.h"
#include "shared.h"
#include "plugins.h"
#include "udaclasses.h"
#include "instrset.h" //Instruction::Scanner()
#include <string>
#include <iomanip>
#include <iostream>
#include <fstream>
using namespace std;

Symbols *symbols=NULL;

//! Looks for appropriate plugin and loads all symbols from file.
void Symbols::Load(const char *filename)
{
    if (filename!=NULL)
    {
        Scanner scanner(filename);
        Scanner::Token t=scanner.Scan();
        Address addr;
        while (t)
        {
            if (t!=tk_number)
                throw Exception(string("number expected in ")+filename+" at "+uint2str(scanner.line()));
            addr=t.int_value;
            t=scanner.Scan();
            if (t!=tk_equal)            
                throw Exception(string("`=' expected in ")+filename+" at "+uint2str(scanner.line()));
            t=scanner.Scan();
            if (t!=tk_text)
                throw Exception(string("\"text\" expected in ")+filename+" at "+uint2str(scanner.line()));
            symbols[addr]=t.str_value;
            t=scanner.Scan();
            if (t!=tk_semicol)
                throw Exception(string("`;' expected in ")+filename+" at "+uint2str(scanner.line()));
            t=scanner.Scan();
        }
    }
    else
    {
        VPlugins::iterator i;
        for (i=plugins.begin();i!=plugins.end();i++)
            if ((*i)->type&PLUGIN_SYMTABLE)
                if ((*i)->IsFileDefined(bfdata.name.c_str()))
                {
                    (*i)->LoadSymbols(*this);
                    break;
                }
    }
}

//! If address is not defined create new label
bool Symbols::checkaddress(const Address addr)
{
    if (!defined(addr))
    {
        symbols[addr]="label"+::uint2str(_counter++);
        return false;
    }
    return true;
}

//! true if symbol for this address is just defined
bool Symbols::defined(const Address addr)
{
    return symbols.find(addr)!=symbols.end();
}

//! for mapping to symbols directly
string &Symbols::operator[](const Address addr)
{
    return symbols[addr];
}


//  find address corresponding to symbol
Address Symbols::findaddress(const string symbol, bool &defined)
{
    MSymbols::iterator i;
    for (i=symbols.begin();i!=symbols.end();i++)
        if (i->second==symbol)
        {
            defined=true;
            return i->first;
        }
    defined=false;
    return 0;
}
