// utils.cc
//
// uda - universal disassembler
// Date: 2003-05-04
//
// Copyright (c) 2002-2003 Ales Smrcka 
//
// This program is licensed under GNU Library GPL. See the file COPYING.
//
// This module contains implementation of some useful functions.
//

#include <string>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include "udaclasses.h"
#include "shared.h"
using namespace std;

string uint2str(unsigned value)
{
    if (value==0) return "0";
    string result;
    unsigned i=1;
    while (i*10<=value) i*=10;
    while (value)
    {
        result += '0'+(value/i);
        value = value - (value/i)*i;
        i /= 10;
    }
    return result;
}

string uint2hex(unsigned value, const string prefix)
{
    if (value==0) return prefix+"0";
    string result;
    char *hex[16]={"0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"};
    while (value)
    {
        result = string(hex[value&0xf]) + result;
        value>>=4;
    }
    return prefix+result;
}

unsigned hex2uint(const string hex)
{
    const char *c=hex.c_str();
    if (hex.substr(0, 2)=="0x")
        c+=2;
    unsigned result=0;
    while (*c)
    {
        if (isdigit(*c))
            result=result<<4 | (*c-'0');
        else
        if (isxdigit(*c))
            result=result<<4 | (lower(*c)-'a'+10);
        else
            return 0;
        c++;
    }
    return result;
}

unsigned number2uint(const string num)
{
    const char *c=num.c_str();
    if (num.substr(0, 2)=="0x")
        return hex2uint(num);
    unsigned result=0;
    while (*c)
    {
        if (isdigit(*c))
            result=result*10 + (*c-'0');
        else
            return 0;
        c++;
    }
    return result;
}

bool directory_exists(const string directory)
{
    DIR *dir;
    if ((dir=opendir(directory.c_str()))!=NULL)
    {
        closedir(dir);
        return true;
    }
    else
        return false;
}

bool file_exists(const string filename)
{
    int fd;
    if ((fd=open(filename.c_str(), O_RDONLY))!=-1)
    {
        close(fd);
        return true;
    }
    else
        return false;
}
